#!/usr/bin/perl -w

package    esmith::FormMagick::Panel::denyhosts;

use strict;
use esmith::ConfigDB;
use esmith::FormMagick;
use esmith::util;
use esmith::cgi;
use File::Basename;
use Exporter;
use Carp;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(get_value get_prop change_settings);

our $VERSION = sprintf '%d.%03d', q$Revision: 1.00 $ =~ /: (\d+).(\d+)/;
our $db = esmith::ConfigDB->open
    || warn "Couldn't open configuration database (permissions problems?)";


sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}

sub get_prop
{
    my $fm   = shift;
    my $item = shift;
    my $prop = shift;

    return $db->get_prop($item, $prop) || '';
}

sub get_value {
    my $fm = shift;
    my $item = shift;
    return ($db->get($item)->value());
}

sub ip_number_or_blank
{
    my $self = shift;
    my $ip = shift;

    if (!defined($ip) || $ip eq "")
    {
        return 'OK';
    }
    return CGI::FormMagick::Validator::ip_number($self, $ip);
}

sub _get_valid_from
{
    my $self = shift;

    my $rec = $db->get('denyhosts');
    return undef unless($rec);
    my @vals = (split ',', ($rec->prop('ValidFrom') || ''));
    return @vals;
}

sub ip_sort(@)
{
    return esmith::util::IPquadToAddr($a) <=> esmith::util::IPquadToAddr($b);
}

sub show_valid_from_list
{
    my $self = shift;
    my $q = $self->{cgi};

    print '<tr><td colspan=2>',$q->p($self->localise('VALIDFROM_DESC')),'</td></tr>';

    my @vals = $self->_get_valid_from();
    if (@vals)
    {
        print '<tr><td colspan=2>',
              $q->start_table({class => "sme-border"}),"\n";
        print $q->Tr(
                esmith::cgi::genSmallCell($q, $self->localise('IP_ADDRESS'),"header"),
                esmith::cgi::genSmallCell($q, $self->localise('REMOVE'),"header"));

        my @cbGroup = $q->checkbox_group(-name => 'validFromRemove',
                -values => [@vals], -labels => { map {$_ => ''} @vals });
        foreach my $val (sort ip_sort @vals)
        {
            print $q->Tr(
                    esmith::cgi::genSmallCell($q, $val, "normal"),
                    esmith::cgi::genSmallCell($q, shift(@cbGroup),
                        "normal"));
        }
        print '</table></td></tr>';
    }
    else
    {
        print $q->Tr($q->td($q->b($self->localise('NO_ENTRIES_YET'))));
    }
    return '';
}

sub show_current_deny
{
    my $self = shift;
    my $q = $self->{cgi};

    print '<tr><td colspan=2>',$q->p($self->localise('CURRENT_DENY_DESC')),'</td></tr>';

    my %vals = ();
    if (open(DENY, "/etc/hosts.deny_ssh") ) {
        %vals = map { m{DenyHosts: (.*) \| (.*)$}; $2 => $1; } grep /DenyHosts:/, <DENY>;
        close DENY;
    }
    if (%vals)
    {
        print '<tr><td colspan=2>',
              $q->start_table({class => "sme-border"}),"\n";
        print $q->Tr(
                esmith::cgi::genSmallCell($q, $self->localise('IP_ADDRESS'),"header"),
                esmith::cgi::genSmallCell($q, $self->localise('FIRST_SEEN'),"header"));

        foreach my $val (sort ip_sort keys %vals)
        {
            print $q->Tr(
                    esmith::cgi::genSmallCell($q, $val, "normal"),
                    esmith::cgi::genSmallCell($q, $vals{$val}, "normal"));
        }
        print '</table></td></tr>';
    }
    else
    {
        print $q->Tr($q->td($q->b($self->localise('NO_ENTRIES_YET'))));
    }
    return '';
}

sub add_new_valid_from
{
    my $self = shift;
    my $q = $self->{cgi};

    my $ip = $q->param('ip');

    # do nothing if no ip was added
    return 1 unless ($ip);

    my $rec = $db->get('denyhosts');
    unless ($rec)
    {
        return $self->error('ERR_NO_RECORD');
    }

    my $prop = $rec->prop('ValidFrom') || '';

    my @vals = split /,/, $prop;
    return '' if (grep /^$ip$/, @vals); # already have this entry

        if ($prop ne '')
        {
            $prop .= ",$ip";
        }
        else
        {
            $prop = "$ip";
        }
    $rec->set_prop('ValidFrom', $prop);
    $q->delete('ip');
    return 1;
}

sub remove_valid_from
{
    my $self = shift;
    my $q = $self->{cgi};

    my @remove = $q->param('validFromRemove');
    my @vals = $self->_get_valid_from();

    foreach my $entry (@remove)
    {
        return undef unless $entry;

        unless (@vals)
        {
            print STDERR "ERROR: unable to load ValidFrom property from conf db\n";
            return undef;
        }

        @vals = (grep { $entry ne $_ } @vals);
    }

    my $prop;
    if (@vals)
    {
        $prop = join ',',@vals;
    }
    else
    {
        $prop = '';
    }
    $db->get('denyhosts')->set_prop('ValidFrom', $prop);
    $q->delete('validFromRemove');

    return 1;
}

sub change_settings {
    my ($fm) = @_;
    my $q = $fm->{'cgi'};

    my %conf;

    # Don't process the form unless we clicked the Save button. The event is
    # called even if we chose the Remove link or the Add link.
    return unless($q->param('Next') eq $fm->localise('SAVE'));

    my $ip = ($q->param ('ip') || '');
    my $status = ($q->param ('status') || 'status');

    #------------------------------------------------------------
    # Looks good; go ahead and change the access.
    #------------------------------------------------------------

    my $rec = $db->get('denyhosts');
    $rec->set_prop('status', $status) if $rec;
    return '' unless $fm->add_new_valid_from;
    return '' unless $fm->remove_valid_from;
    
    unless ( system( "/sbin/e-smith/signal-event", "conf-denyhosts" ) == 0 )
    {
	$fm->error('ERROR_UPDATING');
	return undef;
    }
	
    if ( $rec->prop('status') eq 'disabled' )
    {
        unless ( `/etc/init.d/denyhosts stop`  )
        {
            $fm->error('ERROR_STOPPING');
            return undef;
        }
    }
    
    $fm->success('SUCCESS');
}

1;
